;; -*- emacs-lisp -*-
;;
;; mew-win32-mvm.el --- Movemail interface for Mew-win32.
;;  $Id: mew-win32-mvm.el,v 1.3 1999-08-30 17:29:11+09 shirai Exp $
;;
;;                 Author: "Hideyuki SHIRAI" <Shirai@rdmg.mgcs.mei.co.jp>
;;                Created:             <10/15/1998>
;;                Revised: Time-stamp: <08/30/1999 17:20 shirai>
;;
;; Usage.
;;   Type "M-x mew-win32-mvm-get" or "I" at mew summary mode
;;    , get mail used movemail.
;;
;; Preparation in ~/.emacs.
;;;; mew-win32-mvm (movemail interface for mew-win32)
;; (add-hook 'mew-init-hook
;; 	  (lambda ()
;; 	    (require 'mew-win32-mvm)
;;	    (define-key mew-summary-mode-map "I" 'mew-win32-mvm-get)))
;;
;; (setq mew-win32-mvm-config-list
;;       '(("default"  "default.co.jp" "po:default" "-up")
;;         ("foo"      "foo.org"       "po:foo"  "-upk")))
;;
;;;;  List of movemail parameters,
;;;; 1st cell: Config name (same "case" in .im/Config),
;;;; 2nd cell: Host name,
;;;; 3rd cell: User name,
;;;; 4th cell: Movemail parameters.
;;;;
;;;;  Please read "readme.movemail" in Mule for Win32 archive.
;; 
   
(eval-when-compile
  (require 'mew))

(defconst mew-win32-mvm-version "0.20")

(defvar mew-win32-mvm-config-list
  '(("default"  "default.co.jp" "po:default" "-up")
    ("foo"      "foo.org"       "po:foo"  "-up"))
  "*List of movemail parameters,
1st cell: Config name (same \"case\" in .im/Config),
2nd cell: Host name,
3rd cell: po:User name,
4th cell: Movemail parameters.
\"(car (car mew-win32-mvm-config-list))\" must return \"default\".")

(defvar mew-win32-mvm-after-flush nil
  "*If not nil, flush after movemail")

(defvar mew-win32-mvm-finish-hook nil
  "*Hook called after movemail running.")

;; Internal variables.
(defvar mew-win32-mvm-prog
  (catch 'which
    (let ((file "movemail.exe"))
      (mapcar '(lambda (path)
		 (if (file-executable-p (expand-file-name file path))
		     (throw 'which (expand-file-name file path))))
	      exec-path))))

(cond
 ((boundp 'MULE)
  (defvar mew-cs-win32-mvm '*noconv*))
 (t
  (defvar mew-cs-win32-mvm 'raw-text)))

(defvar mew-win32-mvm-config nil)
(defvar mew-win32-mvm-usr nil)
(defvar mew-win32-mvm-passwd nil)

(defun mew-win32-mvm-get (&optional arg)
  "Get +inbox synchronously ^^; with movemail."
  (interactive "P")
  (let ((mailget nil))
    (if (get-buffer mew-inbox-folder)
	(switch-to-buffer mew-inbox-folder)
      (mew-summary-folder-create mew-inbox-folder))
    (if (not (mew-summary-exclusive-p))
	()
      (if (and mew-summary-trace-directory
	       (not (mew-folder-newsp mew-inbox-folder)))
	  (cd (mew-expand-folder mew-inbox-folder)))
      (mew-summary-folder-cache-manage mew-inbox-folder)
      (if (and mew-summary-cache-use (mew-summary-folder-dir-newp))
	  (progn
	    ;; scan the gap
	    (or arg (goto-char (point-max)))
	    (mew-summary-scan-body mew-prog-imls
				   'mew-summary-mode
				   mew-inbox-folder
				   mew-cs-scan
				   (mew-input-range
				    mew-inbox-folder
				    mew-range-auto-alist
				    mew-ask-range))
	    ;; wait for asynchronous process
	    (while mew-summary-buffer-process
	      (sit-for 1)
	      (discard-input))))
      (set-marker mew-summary-inbox-position (point) (current-buffer))
      ;; for C-xC-x
      (or arg (goto-char (point-max)))
      ;; Flush spool, if mew-win32-mvm-after-flush is nil.
      (if (and (not mew-win32-mvm-after-flush)
	       mew-auto-flush-queue
	       mew-queue-path (file-directory-p mew-queue-path)
	       (> (length (directory-files mew-queue-path)) 2))
	  (mew-summary-flush-queue))
      ;; movemail procedure follow.
      (let ((crashbuf (get-buffer-create "*mew movemail*"))
	    (crashbox (expand-file-name (concat mew-temp-dir "/crashbox"))))
	(save-excursion
	  (set-buffer crashbuf)
	  (erase-buffer))
	(if (file-exists-p crashbox)
	    (delete-file crashbox))
	(setq mailget (mew-win32-mvm-movemail crashbox crashbuf))
	(kill-buffer crashbuf)
	(switch-to-buffer mew-inbox-folder)
	;; Flush spool, if mew-win32-mvm-after-flush is t.
	(if (and mew-win32-mvm-after-flush
		 mew-auto-flush-queue
		 mew-queue-path (file-directory-p mew-queue-path)
		 (> (length (directory-files mew-queue-path)) 2))
	    (progn
	      (mew-summary-flush-queue)
	      (run-hooks 'mew-win32-mvm-finish-hook)))
	(if mailget
	    ;; imget for new mail.
	    (let ((mew-prog-imget-arg-list
		   (list (format "--src=local:%s" crashbox))))
	      (mew-summary-scan-body mew-prog-imget
				     'mew-summary-mode
				     mew-inbox-folder
				     mew-cs-scan)))))))
  
(defun mew-win32-mvm-movemail (crashbox crashbuf)
  "Get mail use movemail."
  (interactive "FCrashbox: \nBCrashbuf: ") ;; for Debug.
  (let (mailget	config-list)
    ;; decide a value that similar .im/Config.
    (if (setq config-list (assoc mew-config-imget mew-win32-mvm-config-list))
	()
      (message "\"%s\" not found in mew-win32-mvm-config-list" mew-config-imget)
      (sit-for 1)
      (message "Try with \"default\".")
      (sit-for 1)
      (if (setq config-list (assoc "default" mew-win32-mvm-config-list))
	  ()
	(error "Something error occor in mew-win32-mvm-config-list !!")))
	
    ;; movemail parameter's setting.
    (setq mew-win32-mvm-config (car config-list))
    (setenv "MAILHOST" (nth 1 config-list))
    (setq mew-win32-mvm-usr (nth 2 config-list))
    (setenv "MOVEMAIL" (nth 3 config-list))
    
    ;; passwd setting.
    (if (nth 4 config-list)
	(setq mew-win32-mvm-passwd (nth 4 config-list))
      (setq mew-win32-mvm-passwd
	    (mew-input-passwd
	     (format "movemail password (%s): " mew-win32-mvm-config))))

    ;; start movemail.
    (if (not (bufferp crashbuf))
	(get-buffer-create crashbuf))
    (message "Getting mail from %s with movemail..." mew-win32-mvm-config)
    (apply (function call-process)
	   mew-win32-mvm-prog
	   nil crashbuf nil
	   (list mew-win32-mvm-usr
		 crashbox
		 mew-win32-mvm-passwd))
    ;; movemail result check.
    (if (not mew-win32-mvm-after-flush)
	(run-hooks 'mew-win32-mvm-finish-hook))
    (save-excursion
      (set-buffer crashbuf)
      (goto-char (point-min))
      (if (not (re-search-forward "[a-z][A-Z]" nil t))
	  (if (file-exists-p crashbox)
	      ;; new mail get.
	      (progn
		(setq mailget t)
		;; crashbox -> mew(mh)
		(mew-win32-mvm-store crashbox)
		(mew-win32-mvm-passwd-save config-list)
		(message "Getting mail from %s with movemail... Done."
			 mew-win32-mvm-config))
	    ;; no mail.
	    (mew-win32-mvm-passwd-save config-list)
	    (message "Getting mail from %s with movemail... No message."
		     mew-win32-mvm-config)
	    (sit-for 1))
	;; something error.
	(message "Getting mail from %s with movemail... Error."
		 mew-win32-mvm-config)
	(sit-for 1)
	(message "Reason: %s" (buffer-substring (point-min)
						(progn
						  (end-of-line) (point))))
	(sit-for 1)))
    mailget))

(defun mew-win32-mvm-passwd-save (config-list)
  (let ((dummy (assoc (car config-list) mew-win32-mvm-config-list)))
    (if (string= mew-win32-mvm-passwd (nth 4 config-list))
	()
      (setq mew-win32-mvm-config-list (delete dummy mew-win32-mvm-config-list))
      (setq mew-win32-mvm-config-list
	    (append mew-win32-mvm-config-list
		    (list (list
			   (car config-list)
			   (nth 1 config-list)
			   (nth 2 config-list)
			   (nth 3 config-list)
			   mew-win32-mvm-passwd)))))))

(defun mew-win32-mvm-store (crashbox)
  "Movemail crashbox to mew inbox."
  (interactive "fCrashbox: ")
  (let ((buf (get-buffer-create "*mew crashbox*"))
	(folder mew-inbox-folder)
	(crend-str (concat (char-to-string ?\015) "$")) ;; ^M
	(msgtop-str (concat (char-to-string ?\014) "\n.*unseen.*\n"))	;; ^L
	(msgend-str (concat "\n" (char-to-string ?\037)))		;; ^_
	(mmdf-separator (concat (make-string 4 ?\001) "\n")))	;; ^A^A^A^A
    (unwind-protect
	(progn
	  (set-buffer buf)
	  (erase-buffer)
	  (condition-case nil
	      (progn
		(mew-frwlet mew-cs-win32-mvm mew-cs-win32-mvm
			    (insert-file-contents crashbox))
		(goto-char (point-min))
		(while (re-search-forward crend-str nil t)
		  (replace-match ""))
		(goto-char (point-min))
		;; from Movemail to MMDF mailbox.
		(while (re-search-forward msgtop-str nil t)
		  (replace-match mmdf-separator)
		  (if (re-search-forward msgend-str nil t)
		      ;; why ?
		      (replace-match (concat "\n" mmdf-separator))))
		(mew-frwlet mew-cs-win32-mvm mew-cs-win32-mvm
			    (write-file crashbox)))
	    (error
	     (message "Movemail: Error occur while crashbox handling.")
	     (sit-for 1)
	     (let ((escape-file (expand-file-name "~/crashbox")))
	       (if (file-exists-p escape-file)
		   (delete-file escape-file))
	       (copy-file crashbox escape-file)
	       (message "Orignal crashbox copy to %s." escape-file)
	       (sit-for 1))))
	  (kill-buffer buf)
	  ))))

(provide 'mew-win32-mvm)
;;; end.
